"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createUnionType = exports.uniqueUnionTypes = exports.createUndefinedType = exports.createStringType = exports.createObjectType = exports.createNumericType = exports.createLiteralType = exports.createInterfaceType = exports.createInstanceOfType = exports.createFunctionType = exports.createElementType = exports.createDOMElementType = exports.createBooleanType = exports.createArrayType = exports.createAnyType = void 0;
const lodash_1 = __importDefault(require("lodash"));
const getTypeHash_1 = __importDefault(require("./getTypeHash"));
function createAnyType(init) {
    return {
        type: 'any',
        jsDoc: init.jsDoc,
    };
}
exports.createAnyType = createAnyType;
function createArrayType(init) {
    return {
        type: 'array',
        jsDoc: init.jsDoc,
        arrayType: init.arrayType,
    };
}
exports.createArrayType = createArrayType;
function createBooleanType(init) {
    return {
        type: 'boolean',
        jsDoc: init.jsDoc,
    };
}
exports.createBooleanType = createBooleanType;
function createDOMElementType(init) {
    return {
        type: 'DOMElementNode',
        jsDoc: init.jsDoc,
        optional: init.optional,
    };
}
exports.createDOMElementType = createDOMElementType;
function createElementType(init) {
    return {
        type: 'ElementNode',
        jsDoc: init.jsDoc,
        elementType: init.elementType,
    };
}
exports.createElementType = createElementType;
function createFunctionType(init) {
    return {
        type: 'FunctionNode',
        jsDoc: init.jsDoc,
    };
}
exports.createFunctionType = createFunctionType;
function createInstanceOfType(init) {
    return {
        type: 'InstanceOfNode',
        instance: init.instance,
        jsDoc: init.jsDoc,
    };
}
exports.createInstanceOfType = createInstanceOfType;
function createInterfaceType(init) {
    return {
        type: 'InterfaceNode',
        jsDoc: init.jsDoc,
        types: init.types ?? [],
    };
}
exports.createInterfaceType = createInterfaceType;
function createLiteralType(init) {
    return {
        type: 'LiteralNode',
        value: init.value,
        jsDoc: init.jsDoc,
    };
}
exports.createLiteralType = createLiteralType;
function createNumericType(init) {
    return {
        type: 'NumericNode',
        jsDoc: init.jsDoc,
    };
}
exports.createNumericType = createNumericType;
function createObjectType(init) {
    return {
        type: 'ObjectNode',
        jsDoc: init.jsDoc,
    };
}
exports.createObjectType = createObjectType;
function createStringType(init) {
    return {
        type: 'StringNode',
        jsDoc: init.jsDoc,
    };
}
exports.createStringType = createStringType;
function createUndefinedType(init) {
    return {
        type: 'UndefinedNode',
        jsDoc: init.jsDoc,
    };
}
exports.createUndefinedType = createUndefinedType;
function uniqueUnionTypes(node) {
    return {
        type: node.type,
        jsDoc: node.jsDoc,
        types: lodash_1.default.uniqBy(node.types, (type) => {
            return (0, getTypeHash_1.default)(type);
        }),
    };
}
exports.uniqueUnionTypes = uniqueUnionTypes;
function createUnionType(init) {
    const flatTypes = [];
    function flattenTypes(nodes) {
        nodes.forEach((type) => {
            if (type.type === 'UnionNode') {
                flattenTypes(type.types);
            }
            else {
                flatTypes.push(type);
            }
        });
    }
    flattenTypes(init.types);
    return uniqueUnionTypes({
        type: 'UnionNode',
        jsDoc: init.jsDoc,
        types: flatTypes,
    });
}
exports.createUnionType = createUnionType;
//# sourceMappingURL=createType.js.map